@@ -5,21 +5,22 @@ import itertools
 
 from suanpan.arguments import Int, String
 from suanpan.proxy import Proxy
+from suanpan import error
 
-try:
-    from suanpan.dw import hive, mysql, postgres # odps
-except ImportError:
-    from suanpan.dw import mock
-    hive = mysql = postgres = mock # = odps
+# try:
+#     from suanpan.dw import hive, mysql, postgres  # odps
+# except ImportError:
+#     from suanpan.dw import mock
+#     hive = mysql = postgres = mock  # odps
 
 
 class DataWarehouseProxy(Proxy):
-    MAPPING = {
-        "hive": hive.DataWarehouse,
-        # "odps": odps.DataWarehouse,
-        "postgres": postgres.DataWarehouse,
-        "mysql": mysql.DataWarehouse,
-    }
+    # MAPPING = {
+    #     "hive": hive.DataWarehouse,
+    #     # "odps": odps.DataWarehouse,
+    #     "postgres": postgres.DataWarehouse,
+    #     "mysql": mysql.DataWarehouse,
+    # }
     DEFAULT_ARGUMENTS = [String("dw-type", default="odps")]
     HIVE_ARGUMENTS = [
         String("dw-hive-host", default="localhost"),
@@ -65,5 +66,24 @@ class DataWarehouseProxy(Proxy):
         )
     )
 
+    def importBackend(self, backendType):
+        try:
+            if backendType == "hive":
+                from suanpan.dw import hive
+                return hive.DataWarehouse
+            elif backendType == "postgres":
+                from suanpan.dw import postgres
+                return postgres.DataWarehouse
+            elif backendType == "mysql":
+                from suanpan.dw import mysql
+                return mysql.DataWarehouse
+            else:
+                raise error.ProxyError(
+                    f"{self.name} don't supported backend type {backendType}"
+                )
+        except ImportError:
+            from suanpan.dw import mock
+            return mock.DataWarehouse
+
 
 dw = DataWarehouseProxy()
