@@ -23,19 +23,19 @@ class Storage(base.Storage):
     PREFERRED_SIZE = 500 * 1024 * 1024  # 500MB
 
     def __init__(
-        self,
-        ossAccessId=None,
-        ossAccessKey=None,
-        ossBucketName="suanpan",
-        ossEndpoint="http://oss-cn-beijing.aliyuncs.com",
-        ossDelimiter="/",
-        ossTempStore=base.Storage.DEFAULT_TEMP_DATA_STORE,
-        ossGlobalStore=base.Storage.DEFAULT_GLOBAL_DATA_STORE,
-        ossDownloadNumThreads=1,
-        ossDownloadStoreName=".py-oss-download",
-        ossUploadNumThreads=1,
-        ossUploadStoreName=".py-oss-upload",
-        **kwargs,
+            self,
+            ossAccessId=None,
+            ossAccessKey=None,
+            ossBucketName="suanpan",
+            ossEndpoint="http://oss-cn-beijing.aliyuncs.com",
+            ossDelimiter="/",
+            ossTempStore=base.Storage.DEFAULT_TEMP_DATA_STORE,
+            ossGlobalStore=base.Storage.DEFAULT_GLOBAL_DATA_STORE,
+            ossDownloadNumThreads=1,
+            ossDownloadStoreName=".py-oss-download",
+            ossUploadNumThreads=1,
+            ossUploadStoreName=".py-oss-upload",
+            **kwargs,
     ):
         super().__init__(
             delimiter=ossDelimiter,
@@ -113,13 +113,13 @@ class Storage(base.Storage):
 
     @runtime.retry(stop_max_attempt_number=3)
     def download(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucketOrBucketName=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucketOrBucketName=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = self.getBucket(bucketOrBucketName)
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -138,14 +138,14 @@ class Storage(base.Storage):
         )
 
     def downloadFolder(
-        self,
-        key,
-        path=None,
-        progress=None,
-        delimiter=None,
-        bucketOrBucketName=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            delimiter=None,
+            bucketOrBucketName=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = self.getBucket(bucketOrBucketName)
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -158,7 +158,7 @@ class Storage(base.Storage):
 
         if self.basename(key) in ignores:
             with spbar.one(
-                total=1, desc="Downloading", disable=quiet, progress=progress
+                    total=1, desc="Downloading", disable=quiet, progress=progress
             ) as pbar:
                 pbar.update(1)
                 pbar.set_description("Ignored")
@@ -194,20 +194,22 @@ class Storage(base.Storage):
             pbar=quiet or not restFiles,
             desc="Removing Rest Files",
         )
-        spath.removeEmptyFolders(path)
+        # only if the `path` folder is getPathInTempStore, try to clear it
+        if path == self.getPathInTempStore(key):
+            spath.removeEmptyFolders(path)
         if not quiet:
             logger.debug(f"Removed empty folders in: {path}")
         # End
         return path
 
     def downloadFile(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucketOrBucketName=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucketOrBucketName=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = self.getBucket(bucketOrBucketName)
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -252,13 +254,13 @@ class Storage(base.Storage):
 
     @runtime.retry(stop_max_attempt_number=3)
     def upload(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucketOrBucketName=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucketOrBucketName=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = self.getBucket(bucketOrBucketName)
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -273,13 +275,13 @@ class Storage(base.Storage):
         )
 
     def uploadFolder(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucketOrBucketName=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucketOrBucketName=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = self.getBucket(bucketOrBucketName)
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -291,7 +293,7 @@ class Storage(base.Storage):
 
         if key in ignores:
             with spbar.one(
-                total=1, desc="Uploading", disable=quiet, progress=progress
+                    total=1, desc="Uploading", disable=quiet, progress=progress
             ) as pbar:
                 pbar.update(1)
                 pbar.set_description("Ignored")
@@ -338,13 +340,13 @@ class Storage(base.Storage):
         return path
 
     def uploadFile(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucketOrBucketName=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucketOrBucketName=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = self.getBucket(bucketOrBucketName)
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -389,7 +391,7 @@ class Storage(base.Storage):
 
     @runtime.retry(stop_max_attempt_number=3)
     def copy(
-        self, name, dist, progress=None, bucketOrBucketName=None, quiet=not g.debug
+            self, name, dist, progress=None, bucketOrBucketName=None, quiet=not g.debug
     ):
         bucket = self.getBucket(bucketOrBucketName)
         copyFunction = (
@@ -402,13 +404,13 @@ class Storage(base.Storage):
         )
 
     def copyFolder(
-        self,
-        src,
-        dist,
-        progress=None,
-        bucketOrBucketName=None,
-        delimiter=None,
-        quiet=not g.debug,
+            self,
+            src,
+            dist,
+            progress=None,
+            bucketOrBucketName=None,
+            delimiter=None,
+            quiet=not g.debug,
     ):
         bucket = self.getBucket(bucketOrBucketName)
         delimiter = delimiter or self.delimiter
@@ -434,7 +436,7 @@ class Storage(base.Storage):
         return dist
 
     def copyFile(
-        self, src, dist, progress=None, bucketOrBucketName=None, quiet=not g.debug
+            self, src, dist, progress=None, bucketOrBucketName=None, quiet=not g.debug
     ):
         bucket = self.getBucket(bucketOrBucketName)
         fileSize = self.getStorageSize(src, bucketOrBucketName=bucket)
@@ -453,13 +455,13 @@ class Storage(base.Storage):
         )
 
     def copySmallFile(
-        self,
-        src,
-        dist,
-        size,
-        progress=None,
-        bucketOrBucketName=None,
-        quiet=not g.debug,
+            self,
+            src,
+            dist,
+            size,
+            progress=None,
+            bucketOrBucketName=None,
+            quiet=not g.debug,
     ):
         bucket = self.getBucket(bucketOrBucketName)
         srcurl = self.storageUrl(src, bucketOrBucketName=bucket)
@@ -469,7 +471,7 @@ class Storage(base.Storage):
             logger.info(f"Copying file: {srcurl} -> {disturl}")
 
         with spbar.one(
-            total=size, desc="Copying", disable=quiet, progress=progress
+                total=size, desc="Copying", disable=quiet, progress=progress
         ) as pbar:
             objectMd5 = self.getStorageMd5(src, bucketOrBucketName=bucket)
             distMd5 = self.getStorageMd5(dist, bucketOrBucketName=bucket)
@@ -484,13 +486,13 @@ class Storage(base.Storage):
             return dist
 
     def copyLargeFile(
-        self,
-        src,
-        dist,
-        size,
-        progress=None,
-        bucketOrBucketName=None,
-        quiet=not g.debug,
+            self,
+            src,
+            dist,
+            size,
+            progress=None,
+            bucketOrBucketName=None,
+            quiet=not g.debug,
     ):
         bucket = self.getBucket(bucketOrBucketName)
         srcurl = self.storageUrl(src, bucketOrBucketName=bucket)
@@ -500,7 +502,7 @@ class Storage(base.Storage):
             logger.info(f"Copying file: {srcurl} -> {disturl}")
 
         with spbar.one(
-            total=size, desc="Copying", disable=quiet, progress=progress
+                total=size, desc="Copying", disable=quiet, progress=progress
         ) as pbar:
             objectMd5 = self.getStorageMd5(src, bucketOrBucketName=bucket)
             distMd5 = self.getStorageMd5(dist, bucketOrBucketName=bucket)
@@ -535,12 +537,12 @@ class Storage(base.Storage):
 
     @runtime.retry(stop_max_attempt_number=3)
     def remove(
-        self,
-        key,
-        progress=None,
-        delimiter=None,
-        bucketOrBucketName=None,
-        quiet=not g.debug,
+            self,
+            key,
+            progress=None,
+            delimiter=None,
+            bucketOrBucketName=None,
+            quiet=not g.debug,
     ):
         delimiter = delimiter or self.delimiter
         bucket = self.getBucket(bucketOrBucketName)
@@ -558,12 +560,12 @@ class Storage(base.Storage):
         )
 
     def removeFolder(
-        self,
-        key,
-        progress=None,
-        delimiter=None,
-        bucketOrBucketName=None,
-        quiet=not g.debug,
+            self,
+            key,
+            progress=None,
+            delimiter=None,
+            bucketOrBucketName=None,
+            quiet=not g.debug,
     ):
         delimiter = delimiter or self.delimiter
         bucket = self.getBucket(bucketOrBucketName)
@@ -587,12 +589,12 @@ class Storage(base.Storage):
         return key
 
     def removeFile(
-        self,
-        key,
-        progress=None,
-        delimiter=None,
-        bucketOrBucketName=None,
-        quiet=not g.debug,
+            self,
+            key,
+            progress=None,
+            delimiter=None,
+            bucketOrBucketName=None,
+            quiet=not g.debug,
     ):  # pylint: disable=unused-argument
         delimiter = delimiter or self.delimiter
         bucket = self.getBucket(bucketOrBucketName)
@@ -602,7 +604,7 @@ class Storage(base.Storage):
             logger.info(f"Removing file: {url}")
 
         with spbar.one(
-            total=1, disable=quiet, desc="Removing", progress=progress,
+                total=1, disable=quiet, desc="Removing", progress=progress,
         ) as pbar:
             bucket.delete_object(key)
             pbar.update(1)
@@ -616,8 +618,8 @@ class Storage(base.Storage):
         objects = (
             obj
             for obj in runtime.saferun(self._listAll, default=iter(()))(
-                key, delimiter=delimiter, bucketOrBucketName=bucket
-            )
+            key, delimiter=delimiter, bucketOrBucketName=bucket
+        )
             if obj.key != root
         )
         folders, files = lcc.divide(objects, lambda obj: obj.is_prefix())
@@ -633,24 +635,24 @@ class Storage(base.Storage):
         return (
             obj
             for obj in self.autoRefreshToken(oss2.ObjectIterator)(
-                delimiter=delimiter, prefix=prefix, bucket=bucket
-            )
+            delimiter=delimiter, prefix=prefix, bucket=bucket
+        )
         )
 
     def listAll(self, key, delimiter=None, bucketOrBucketName=None):
         return (
             obj.key
             for obj in self._listAll(
-                key, delimiter=delimiter, bucketOrBucketName=bucketOrBucketName
-            )
+            key, delimiter=delimiter, bucketOrBucketName=bucketOrBucketName
+        )
         )
 
     def listFolders(self, key, delimiter=None, bucketOrBucketName=None):
         return (
             obj.key
             for obj in self._listAll(
-                key, delimiter=delimiter, bucketOrBucketName=bucketOrBucketName
-            )
+            key, delimiter=delimiter, bucketOrBucketName=bucketOrBucketName
+        )
             if obj.is_prefix()
         )
 
@@ -658,8 +660,8 @@ class Storage(base.Storage):
         return (
             obj.key
             for obj in self._listAll(
-                key, delimiter=delimiter, bucketOrBucketName=bucketOrBucketName
-            )
+            key, delimiter=delimiter, bucketOrBucketName=bucketOrBucketName
+        )
             if not obj.is_prefix()
         )
 
@@ -671,6 +673,30 @@ class Storage(base.Storage):
         bucket = self.getBucket(bucketOrBucketName)
         return bucket.object_exists(key)
 
+    def presignedGetObject(self, key, expires=3600, bucketOrBucketName=None, slash_safe=False):
+        bucket = self.getBucket(bucketOrBucketName)
+        url = bucket.sign_url('GET', key, expires, slash_safe=slash_safe)
+        return url
+
+    def presignedPutObject(self, key, expires=3600, bucketOrBucketName=None, slash_safe=False):
+        bucket = self.getBucket(bucketOrBucketName)
+        url = bucket.sign_url('PUT', key, expires, slash_safe=slash_safe)
+        return url
+
+    def getObject(self, key, bucket=None, **kwargs):
+        logger.debug(f"Get object from {key}")
+        bucket = self.getBucket(bucket)
+
+        object_stream = self.autoRefreshToken(bucket.get_object)(key, **kwargs)
+        return object_stream.read()
+
+    def putObject(self, key, data, bucket=None, **kwargs):
+        logger.debug(f"Put object to {key}")
+        bucket = self.getBucket(bucket)
+
+        result = self.autoRefreshToken(bucket.put_object)(key, data, **kwargs)
+        return result
+
     def getStorageMd5(self, key, bucketOrBucketName=None):
         bucket = self.getBucket(bucketOrBucketName)
         try:
